package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.container.StreamProbeResult;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import net.knarcraft.ffmpegconverter.utility.FileUtil;
import net.knarcraft.ffmpegconverter.utility.OutputUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.File;
import java.io.IOException;

/**
 * Implements all methods which can be useful for any implementation of a converter.
 */
public abstract class AbstractConverter implements Converter {

    final boolean debug = false;
    private final String newExtension;
    String ffprobePath;
    String ffmpegPath;
    String[] audioFormats;
    String[] videoFormats;

    /**
     * Initializes variables used by the abstract converter
     */
    AbstractConverter(@Nullable String newExtension) {
        this.newExtension = newExtension;
        OutputUtil.setDebug(this.debug);
        try {
            audioFormats = FileUtil.readFileLines("audio_formats.txt");
            videoFormats = FileUtil.readFileLines("video_formats.txt");
        } catch (IOException e) {
            System.out.println("Unable to read audio and/or video formats from internal files.");
            System.exit(1);
        }
    }

    /**
     * Reads streams from a file, and converts it to a mp4 file
     *
     * @param folder <p>The folder of the file to process.</p>
     * @param file   <p>The file to process.</p>
     * @throws IOException <p>If the BufferedReader fails.</p>
     */
    private void processFile(@NotNull File folder, @NotNull File file) throws IOException {
        StreamProbeResult probeResult = FFMpegHelper.probeFile(ffprobePath, file);
        if (probeResult.parsedStreams().isEmpty()) {
            throw new IllegalArgumentException("The file has no valid streams. Please make sure the file exists and" +
                    " is not corrupt.");
        }

        String outExtension = newExtension != null ? newExtension : FileUtil.getExtension(file.getName());
        String newPath = FileUtil.getNonCollidingPath(folder, file, outExtension);
        OutputUtil.println();
        OutputUtil.println("Preparing to start process...");
        OutputUtil.println("Converting " + file);

        String[] command = generateConversionCommand(ffmpegPath, probeResult, newPath);
        // If no commands were given, no conversion is necessary
        if (command.length == 0) {
            return;
        }

        ProcessBuilder processBuilder = new ProcessBuilder(command);
        FFMpegHelper.runProcess(processBuilder, folder, "\n", true);
    }

    @Override
    public void convert(@NotNull File file) throws IOException {
        processFile(file.getParentFile(), file);
    }

}
